VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cFontAwesomeSVG"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit 'a Class, which deals with the Archive-Content, allowing access to the SVGs per Key

Public DefaultSize As Long

Private WAC As cWebArchive, SVG As cSVG, Cache As cCollection

Private Sub Class_Initialize()
  ' CreateWACFileFromDirectory App.Path & "\SVGs", App.Path & "\_FontAwesomeSVGs.wac"
  DefaultSize = 64 'pre-define the DefaultSize as 64x64 Pixels
  Set Cache = New_c.Collection(False)
  Set WAC = New_c.WebArchive(App.Path & "\_FontAwesomeSVGs.wac")  'read the Archive-Content from Disk into Memory
  Set SVG = New_c.SVG 'instantiate the SVG-Renderer
  ' ReCreateSVGFolderFromArchive
End Sub

Public Property Get KeyList() As String 'hands out a vbCrLf-separated String-List with the Icon-Keys
  If WAC.Exists("KeyList.txt") Then KeyList = Cairo.ToBSTR(WAC.GetContent("KeyList.txt"))
End Property

Public Function GetSurface(Key As String, Optional ByVal ForeColor As Long, Optional ByVal DesiredWidth As Long, Optional ByVal DesiredHeight As Long) As cCairoSurface
Dim CacheKey As String, Srf As cCairoSurface
Const F As Double = 0.1
  If DesiredWidth <= 0 Then DesiredWidth = DefaultSize
  If DesiredHeight <= 0 Then DesiredHeight = DefaultSize
  If DesiredWidth > 512 Then DesiredWidth = 512
  If DesiredHeight > 512 Then DesiredHeight = 512
  
  If WAC.Exists(Key & ".svg") Then
    CacheKey = Key & "_" & ForeColor & "_" & DesiredWidth & "x" & DesiredHeight
    If Not Cache.Exists(CacheKey) Then
      Set Srf = Cairo.CreateSurface(DesiredWidth, DesiredHeight)
      SVG.ParseContent WAC.GetContent(Key & ".svg")
      SVG.RenderFromDOM Srf.CreateContext, 0.5, 0.5, Srf.Width - 1, Srf.Height - 1, 0, True
      If ForeColor Then
        With Cairo.CreateSurface(DesiredWidth, DesiredHeight).CreateContext
          .SetSourceColor ForeColor
          .MaskSurface Srf
          Set Srf = .Surface
        End With
      End If
      Cache.Add Srf, CacheKey
    End If
    Set GetSurface = Cache(CacheKey)
  Else
    Set GetSurface = CreateNonExistingSurface(DesiredWidth, DesiredHeight)
  End If
End Function

Public Function GetPicture(Key As String, Optional ByVal ForeColor As Long, Optional ByVal DesiredWidth As Long, Optional ByVal DesiredHeight As Long, Optional ByVal BackColor As Long = vbWhite) As StdPicture
  Dim Srf As cCairoSurface, CC As cCairoContext
  Set Srf = GetSurface(Key, ForeColor, DesiredWidth, DesiredHeight)
  Set CC = Cairo.CreateSurface(Srf.Width, Srf.Height).CreateContext
      CC.SetSourceColor BackColor
      CC.Paint
      CC.RenderSurfaceContent Srf, 0, 0
  Set GetPicture = CC.Surface.Picture
End Function

Private Function CreateNonExistingSurface(ByVal DesiredWidth As Long, ByVal DesiredHeight As Long) As cCairoSurface
Dim CacheKey As String, Size As Long
  CacheKey = "NotExisting_" & DesiredWidth & "x" & DesiredHeight
  If Not Cache.Exists(CacheKey) Then
    With Cairo.CreateSurface(DesiredWidth, DesiredHeight).CreateContext
      Size = IIf(DesiredWidth < DesiredHeight, DesiredWidth, DesiredHeight)
      .SetLineWidth Size * 0.02
      .SetSourceColor &H444444
      .Rectangle 0, 0, .Surface.Width, .Surface.Height, True
      .Stroke
      .TranslateDrawings DesiredWidth / 2, DesiredHeight / 2
      .RotateDrawingsDeg 45
      
      .DrawLine -Size / 2, 0, Size / 2, 0, , Size * 0.12, vbRed
      .DrawLine 0, -Size / 2, 0, Size / 2, , Size * 0.12, vbRed
      Cache.Add .Surface, CacheKey
    End With
  End If
  Set CreateNonExistingSurface = Cache(CacheKey)
End Function
 
'the Helper-Function which was used to create the WAC-file, this class is loading in Class_Initialize
Private Sub CreateWACFileFromDirectory(SVGFolder As String, WacFileName As String)
  Dim DL As cDirList, i As Long
  Set DL = New_c.FSO.GetDirList(SVGFolder, dlSortByNameLogically, "*.svg")
  With New_c.StringBuilder
    For i = 0 To DL.FilesCount - 1
      .Append IIf(i, vbCrLf, "") & Left(DL.FileName(i), Len(DL.FileName(i)) - 4)
    Next
    New_c.FSO.WriteTextContent SVGFolder & "\KeyList.txt", .ToString
  End With
  With New_c.WebArchive
    .ReadContentsFromDirectory SVGFolder
    .SaveContentsToArchiveFile WacFileName
  End With
End Sub

'one can re-establish the Folder (with the original FontAwesome-SVG-Files) on disk, by calling the following routine
Private Sub ReCreateSVGFolderFromArchive()
  WAC.WriteContentsToDirectory App.Path & "\SVGs"
End Sub

