VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cwColorChooser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit 'a circular ColorChooser (implemented as an RC5 cwWidget-Class)

Event ColorChanged()

Const RingSurfaceSize& = 384, RingWidth& = 42 '<- that's the consts for the OneTime-Surface-construction

Private xCenter As Single, yCenter As Single, RingRadius As Single
Private RingSrf As cCairoSurface, CurRingIdx As Long, ColorLut(0 To 1440 - 1) As Long

Private WithEvents W As cWidgetBase
Attribute W.VB_VarHelpID = -1

Private Sub Class_Initialize()
  Set W = Cairo.WidgetBase
      W.BackColor = vbWhite
      W.ImplementsWheelMessages = True
  Set RingSrf = CreateRingSpectrumSurface(RingSurfaceSize, RingWidth)
End Sub
 
Public Property Get Widget() As cWidgetBase
  Set Widget = W
End Property
Public Property Get Widgets() As cWidgets
  Set Widgets = W.Widgets
End Property

Public Property Get Color() As Long
  Color = ColorLut(CurRingIdx)
End Property

Private Sub W_MouseWheel(ByVal MouseKeys As Long, ByVal Rotation As Long, ByVal LineOffset As Long, ByVal xAbs As Single, ByVal yAbs As Single)
  CurRingIdx = (CurRingIdx + UBound(ColorLut) + 1 + LineOffset * 8) Mod UBound(ColorLut) + 1
  W.Refresh: RaiseEvent ColorChanged
End Sub

Private Sub W_Resize()
  xCenter = W.ScaleWidth / 2: yCenter = W.ScaleHeight / 2
  RingRadius = W.ScaleWidth * 0.4
  W.Refresh
End Sub

Private Sub W_MouseMove(Button As Integer, Shift As Integer, ByVal X As Single, ByVal Y As Single)
  If Button <> 1 Then Exit Sub
  CurRingIdx = (Cairo.CalcArc(yCenter - Y, xCenter - X) * 720 / Cairo.PI + 1080) Mod 1440
  W.Refresh: RaiseEvent ColorChanged
End Sub

Private Sub W_Paint(CC As cCairoContext, ByVal xAbs As Single, ByVal yAbs As Single, ByVal dx_Aligned As Single, ByVal dy_Aligned As Single, UserObj As Object)
  DrawBorderAndBackGround CC, dx_Aligned, dy_Aligned

  CC.ARC xCenter, yCenter, 0.4 * RingRadius 'the centered Circle-Fill with the current color
    CC.SetSourceColor ColorLut(CurRingIdx)
    CC.Fill True
    
    CC.SetLineWidth 2.8
    CC.SetSourceColor vbBlack, 0.25
    CC.Stroke True 'intermediate stroke (the path is still there)

    CC.SetLineWidth 1.25
    CC.SetSourceColor vbWhite, 0.7
  CC.Stroke 'final stroke

  CC.Save
    CC.TranslateDrawings xCenter + 0.5, yCenter + 0.5
    CC.SetSourceColor ColorLut(CurRingIdx)
    
    'the RingSurface
    CC.RenderSurfaceContent RingSrf, -RingRadius, -RingRadius, 2 * RingRadius, 2 * RingRadius
    
    'the small triangle (+ the line leading to it)
    CC.SetLineWidth 0.4
    CC.RotateDrawingsDeg CurRingIdx / 4
      CC.MoveTo 0, 0
      CC.LineTo 0, -RingRadius - 0.75
      CC.RelLineTo -8, -8
      CC.RelLineTo 16, 0
      CC.RelLineTo -8, 8
    CC.Fill True
    CC.Stroke
    
    'the small black/white "inner-circle"
    Dim ScaledRW#: ScaledRW = RingWidth * RingRadius / (RingSurfaceSize - RingWidth - 6)
    CC.ARC 0, -RingRadius + ScaledRW, ScaledRW * 0.6
      CC.SetLineWidth 3
      CC.SetSourceColor vbBlack, 0.55
      CC.Stroke True 'intermediate stroke (the path is still there)

      CC.SetLineWidth 1.25
      CC.SetSourceColor vbWhite, 0.7
    CC.Stroke 'final stroke
  CC.Restore
End Sub

Private Sub DrawBorderAndBackGround(CC As cCairoContext, dx, dy)
  CC.SetLineWidth 1, True
  CC.Rectangle 0, 0, dx, dy, True
    CC.SetSourceColor W.BackColor, W.Alpha
  CC.Fill True
    CC.SetSourceColor W.BorderColor, W.Alpha
  CC.Stroke
End Sub

Private Function CreateRingSpectrumSurface(Size, RingWidth) As cCairoSurface
Dim i&, Lst As cArrayList, LUTSrf As cCairoSurface, LUT() As Byte
  Set LUTSrf = Cairo.CreateSurface(1440, 1) 'a long, but only one Pixel high "Stripe-Surface" (1/4 of a degree -> 360*4=1440)
  With Cairo.CreateLinearPattern(0, 0, LUTSrf.Width - 1, 0)
    Set Lst = New_c.ArrayList(vbLong, vbRed, vbYellow, vbGreen, vbCyan, vbBlue, vbMagenta, vbRed)
    For i = 0 To Lst.Count - 1
      .AddColorStop i / 6, Lst(i)
    Next i
    LUTSrf.CreateContext.Paint 1, .This
  End With
  
  Set CreateRingSpectrumSurface = Cairo.CreateSurface(Size, Size)
  With CreateRingSpectrumSurface.CreateContext
   .SetLineCap CAIRO_LINE_CAP_ROUND
   .SetLineWidth (Size - RingWidth - 3.5) * 2 * Cairo.PI / LUTSrf.Width
   .TranslateDrawings Size \ 2, Size \ 2
    LUTSrf.BindToArray LUT
    For i = 0 To 1440 - 1
      ColorLut(i) = RGB(LUT(i * 4 + 2, 0), LUT(i * 4 + 1, 0), LUT(i * 4, 0))
      .SetSourceColor ColorLut(i)
        .MoveTo 0, -(Size / 2 - RingWidth - 3.5)
        .RelLineTo 0, -RingWidth
      .Stroke
      .RotateDrawingsDeg 0.25 'rotate by one quarter of a degree (360 / (1440 steps in the LUT))
    Next i
    LUTSrf.ReleaseArray LUT
  End With
End Function
